#pragma once
#include <GL/glew.h>
#include <glm/glm.hpp>
#include <vector>

#include "Components/TilemapComponent.h"
#include "Components/SpriteComponent.h"
#include "utils/EngineConfig.h"

struct Light {
    glm::vec2 screenPos;
    glm::vec3 color;
    float intensity;
    float radius;
};

class Renderer {
public:
    static void Init();
    static void Resize(int w, int h);
    static void Begin();
    static void End();

    static void DrawSprite(SpriteComponent* sprite, const glm::vec2& pos, float zoom, glm::vec2& CameraPos);
    static void DrawTilemap(TilemapComponent* tilemap, const glm::vec2& worldPos, float zoom, const glm::vec2& cameraPos);

    static void AddLight(const glm::vec2& screenPos, const glm::vec3& color, float intensity, float radius);
    static void ClearLights();

    static void DrawEditorGrid(const glm::vec2& cameraPos, float zoom);

    // —— New gizmo functions —— 
    // Draws a colored line between two world‑space points.
    static void DrawGizmoLine(
        const glm::vec2& worldStart,
        const glm::vec2& worldEnd,
        const glm::vec3& color,
        const glm::vec2& cameraPos,
        float zoom
    );

    // Draws a colored axis‑aligned rectangle in world space.
    static void DrawGizmoRect(
        const glm::vec2& worldPos,
        const glm::vec2& size,
        const glm::vec3& color,
        const glm::vec2& cameraPos,
        float zoom
    );

    // Draws a colored circle (approximated by segments) in world space.
    static void DrawGizmoCircle(
        const glm::vec2& worldCenter,
        float radius,
        int segments,
        const glm::vec3& color,
        const glm::vec2& cameraPos,
        float zoom
    );

    static GLuint GetRenderTexture();
    static glm::ivec2 GetSize();
    static int GetDrawCallCount();
    static int GetLightsCount();

private:
    static std::vector<Light> s_Lights;
    static GLuint fbo, textureColorBuffer, rbo;
    static GLuint defaultNormalMap;
    static int width, height;
    static int s_DrawCalls;
    static int s_LightsCount;

    static GLuint shader, quadVAO, quadVBO;
    static void InitQuad();
    static GLuint LoadShader(const char* vertexSrc, const char* fragmentSrc);
};
